pro DataPrepareSJI, fuvSJI1, fuvSJI2, nuvSJI1, nuvSJI2, rasterSJI, CCDx, CCDy, timefac, PZT, SJIData, SJIind, SJI, $
    threshold_SJI_Overview, threshold_SJI
    
  ; INPUT:
  ; fuvSJI1: input data from SIM, containing FUV SJI time t
  ; fuvSJI2: input data from SIM, containing FUV SJI time t+1
  ; nuvSJI1: input data from SIM, containing NUV SJI time t
  ; nuvSJI2: input data from SIM, containing NUV SJI time t+1
  ; rasterSJI: which wavelength should be used for overview graph ('fu1, 'fu2', 'nu1', 'nu2')
  ; CCDx: size of detector in pixels in x-direction
  ; CCDy: size of detector in pixels in y-direction
  ; timefac: factor between t and t+1 (((t+1)-t)/dt)
  ; PZT: the current PZT values
  ; SJIData: created by initgraph.pro, will be changed, i.e. data is updated
  ; SJIind: the index of the current SJI
  ;
  ; OUTPUT:
  ; SJI: the resulting SJI
  ;
  ; created by Martin Wiesmann (UIO), Sept 2012
    
    
  ;SJI calculation
  ;Overview plot
  ;interpolate SJI and get right SJI
  case rasterSJI of
    'fu1': BEGIN
      UV=1
      SJI = timefac * fuvSJI2[*,*,0] + (1-timefac) * fuvSJI1[*,*,0]
    end
    'fu2': BEGIN
      UV=1
      SJI = timefac * fuvSJI2[*,*,1] + (1-timefac) * fuvSJI1[*,*,1]
    end
    'nu1': BEGIN
      UV=0
      SJI = timefac * nuvSJI2 + (1-timefac) * nuvSJI1
    end
    'nu2': BEGIN    ;at the moment I have only one nuv SJI (NUV1)
      UV=0
      SJI = timefac * nuvSJI2 + (1-timefac) * nuvSJI1
    end
    else: BEGIN
      UV = -1
      SJI = fltarr(SJIData[0].Boxcoords[4]+1, SJIData[0].Boxcoords[5]+1)
    end
  endcase
  
  if UV ge 0 then begin
    sSJI = size(SJI)
    
    ;simulation has half the resolution in x-direction, so interpolate missing values (only FUV)
    if UV then begin
      SJI=interpolate(SJI,findgen(sSJI[1]*2)/2,findgen(sSJI[2]),/grid)
      sSJI = size(SJI)
    endif
    
    
    ;tile SJI as well
    ;we only calculate the area which we actually need, note this might be bigger than the actual detector due to PZT movements
    CCDxtemp=SJIData[0].Boxcoords[4]+1
    CCDytemp=SJIData[0].Boxcoords[5]+1
    rSJI = make_array(CCDxtemp, CCDytemp, /float)
    keepcopying=1
    low=0
    high=low+sSJI[2]
    while keepcopying do begin      ;vertical
      if low ge CCDytemp then begin
        keepcopying=0
      endif else begin
        if high gt CCDytemp-1 then begin
          miss=CCDytemp-1-low
          rSJI[0:sSJI[1]-1,low:CCDytemp-1] = SJI[*,0:miss]
          keepcopying=0
        endif else begin
          rSJI[0:sSJI[1]-1,low:high-1] = SJI
          low=low+sSJI[2]
          high=high+sSJI[2]
        endelse
      endelse
    endwhile
    keepcopying=1
    low=sSJI[1]
    high=low+sSJI[1]
    while keepcopying do begin      ;horizontal
      if low ge CCDxtemp then begin
        keepcopying=0
      endif else begin
        if high gt CCDxtemp-1 then begin
          miss=CCDxtemp-1-low
          rSJI[low:CCDxtemp-1,*] = rSJI[0:miss,*]
          keepcopying=0
        endif else begin
          rSJI[low:high-1,*] = rSJI[0:sSJI[1]-1,*]
          low=low+sSJI[1]
          high=high+sSJI[1]
        endelse
      endelse
    endwhile
    
  endif
  ;finally, set the data
  *(SJIData[0]).data = rSJI[(SJIData[0].Boxcoords[0]):(SJIData[0].Boxcoords[4]), SJIData[0].Boxcoords[2]:SJIData[0].Boxcoords[5]]
  
  ;the first time this spectral window is populated, we need to set the minium and maximum value for scaling
  if (SJIData[0].min_show eq 0) && (SJIData[0].max_show eq 0) then begin
    temp = Histo_Opt(*(SJIData[0]).data, threshold_SJI_Overview)
    SJIData[0].max_show = max(temp, min=m)
    SJIData[0].min_show = m
  endif
  
  
  
  
  
  
  ;real SJI, only when there is actually an SJI taken
  if SJIind ge 0 then begin
  
    ;first we set the box of the overview SJI to the box of the current SJI
    SJIData[0].Showcoords = SJIData[SJIind].Boxcoords
    for i=0,5 do $
      if SJIData[0].Showcoords[i] gt CCDx then SJIData[0].Showcoords[i] = SJIData[0].Showcoords[i] - CCDx
      
    case SJIData[SJIind].type of
      'FUV1': BEGIN
        UV=1
        SJI = timefac * fuvSJI2[*,*,0] + (1-timefac) * fuvSJI1[*,*,0]
      end
      'FUV2': BEGIN
        UV=1
        SJI = timefac * fuvSJI2[*,*,1] + (1-timefac) * fuvSJI1[*,*,1]
      end
      'NUV1': BEGIN
        UV=0
        SJI = timefac * nuvSJI2 + (1-timefac) * nuvSJI1
      end
      'NUV2': BEGIN    ;at the moment I have only one nuv SJI (NUV1)
        UV=0
        SJI = timefac * nuvSJI2 + (1-timefac) * nuvSJI1
      end
      else: BEGIN
        UV=-1
        SJI = fltarr(CCDx,CCDy)
      end
    endcase
    
    
    rSJI = rSJI*0
    
    if UV ge 0 then begin
      sSJI = size(SJI)
      
      ;simulation has half the resolution in x-direction, so interpolate missing values (only FUV)
      if UV then begin
        SJI=interpolate(SJI,findgen(sSJI[1]*2)/2,findgen(sSJI[2]),/grid)
        sSJI = size(SJI)
      endif
      
      ;before tiling, we shift the SJI, according to PZT
      x = PZT[0]*6 mod sSJI[1]
      if x ne 0 then begin
        if x lt 0 then x=sSJI[1]+x
        x1 = floor(x)
        spacefacx = x-x1
        
        ;shift
        SJItemp = fltarr(sSJI[1], sSJI[2])
        SJItemp[0:sSJI[1]-x1-1,*] = SJI[x1:sSJI[1]-1,*]
        SJItemp[sSJI[1]-x1:sSJI[1]-1,*] = SJI[0:x1-1,*]
        SJI = SJItemp
        ;interpolate
        SJItemp = fltarr(sSJI[1]+1, sSJI[2])
        SJItemp[0:sSJI[1]-1,*] = SJI
        SJItemp[sSJI[1],*] = SJI[0,*]
        SJI = interpolate(SJItemp, findgen(sSJI[1])+spacefacx, findgen(sSJI[2]), /grid)
      endif
      
      
      y = PZT[1]*6 mod sSJI[2]
      if y ne 0 then begin
        if y lt 0 then y=sSJI[2]+y
        y1 = floor(y)
        spacefacy = y-y1
        
        ;shift
        SJItemp = fltarr(sSJI[1], sSJI[2])
        SJItemp[*,0:sSJI[2]-y1-1] = SJI[*,y1:sSJI[2]-1]
        SJItemp[*,sSJI[2]-y1:sSJI[2]-1] = SJI[*,0:y1-1]
        SJI = SJItemp
        ;interpolate
        SJItemp = fltarr(sSJI[1], sSJI[2]+1)
        SJItemp[*,0:sSJI[2]-1] = SJI
        SJItemp[*,sSJI[1]] = SJI[*,0]
        SJI = interpolate(SJItemp, findgen(sSJI[1]), findgen(sSJI[2])+spacefacy, /grid)
      endif
      
      
      ;tile SJI as well
      ;we only calculate the area which we actually need, note this might be bigger than the actual detector due to PZT movements
      CCDxtemp=SJIData[SJIind].Boxcoords[4]+1
      CCDytemp=SJIData[SJIind].Boxcoords[5]+1
      rSJI = make_array(CCDxtemp, CCDytemp, /float)
      keepcopying=1
      low=0
      high=low+sSJI[2]
      while keepcopying do begin      ;vertical
        if low ge CCDytemp then begin
          keepcopying=0
        endif else begin
          if high gt CCDytemp-1 then begin
            miss=CCDytemp-1-low
            rSJI[0:sSJI[1]-1,low:CCDytemp-1] = SJI[*,0:miss]
            keepcopying=0
          endif else begin
            rSJI[0:sSJI[1]-1,low:high-1] = SJI
            low=low+sSJI[2]
            high=high+sSJI[2]
          endelse
        endelse
      endwhile
      keepcopying=1
      low=sSJI[1]
      high=low+sSJI[1]
      while keepcopying do begin      ;horizontal
        if low ge CCDxtemp then begin
          keepcopying=0
        endif else begin
          if high gt CCDxtemp-1 then begin
            miss=CCDxtemp-1-low
            rSJI[low:CCDxtemp-1,*] = rSJI[0:miss,*]
            keepcopying=0
          endif else begin
            rSJI[low:high-1,*] = rSJI[0:sSJI[1]-1,*]
            low=low+sSJI[1]
            high=high+sSJI[1]
          endelse
        endelse
      endwhile
      
    endif
    ; set all data to zero, except for CRS windows
    startR = SJIData[SJIind].Boxcoords[0]
    endR = SJIData[SJIind].Boxcoords[4]
    startC = SJIData[SJIind].Boxcoords[2]
    endC = SJIData[SJIind].Boxcoords[5]
    SJI = fltarr(CCDx, CCDy)
    SJI[startR:endR, startC:endC] = rSJI[startR:endR, startC:endC]
    
    ;check if we need to rebin the data
    sumX=SJIData[SJIind].sum_spatX
    sumY=SJIData[SJIind].sum_spatY
    if (sumX gt 1) || (sumY gt 1) then begin
      if sumX eq 8 then $
        SJI = congrid(SJI, CCDx/sumX, CCDy/sumY, /interp) * sumX * sumY $
      else SJI = rebin(SJI, CCDx/sumX, CCDy/sumY) * sumX * sumY
    endif
    
    ;adjust for rebinning
    if sumX gt 1 then begin
      startR = startR / sumX
      endR = endR / sumX
    endif
    if sumY gt 1 then begin
      startC = startC / sumY
      endC = endC / sumY
    endif
    
    ;multiply with defined exposure duratioin
    SJI = SJI * SJIData[SJIind].Exp_Duration/1000d
    
    ;finally, set the data
    *(SJIData[SJIind]).data = SJI[startR:endR, startC:endC]
    
    ;the first time this spectral window is populated, we need to set the minium and maximum value for scaling
    if (SJIData[SJIind].min_show eq 0) && (SJIData[SJIind].max_show eq 0) then begin
      temp = Histo_Opt(*(SJIData[SJIind]).data, threshold_SJI)
      SJIData[SJIind].max_show = max(temp, min=m)
      SJIData[SJIind].min_show = m
    endif
    
    ;we save the PZT offset for each SJI indivually, for the axis of the plot
    SJIData[SJIind].PZTx = PZT[0]
    SJIData[SJIind].PZTy = PZT[1]
  endif
end
